from typing import Any

from ..objects.payment_link import PaymentLink
from .base import ResourceCreateMixin, ResourceGetMixin, ResourceListMixin

__all__ = [
    "PaymentLinks",
]


class PaymentLinks(ResourceCreateMixin, ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/payment_links` endpoint."""

    RESOURCE_ID_PREFIX: str = "pl_"

    def get_resource_path(self) -> str:
        return "payment-links"

    def get_resource_object(self, result: dict) -> PaymentLink:
        return PaymentLink(result, self.client)

    def get(self, resource_id: str, **params: Any) -> PaymentLink:
        self.validate_resource_id(resource_id, "payment link ID")
        return super().get(resource_id, **params)



from typing import TYPE_CHECKING, Any, Dict, Optional

from ..objects.order import Order
from ..objects.shipment import Shipment
from .base import ResourceCreateMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin

if TYPE_CHECKING:
    from ..client import Client

__all__ = [
    "OrderShipments",
]


class OrderShipments(ResourceCreateMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin):
    """Resource handler for the `/orders/:order_id:/shipments` endpoint."""

    RESOURCE_ID_PREFIX: str = "shp_"

    _order: Order

    def __init__(self, client: "Client", order: Order) -> None:
        self._order = order
        super().__init__(client)

    def get_resource_object(self, result: dict) -> Shipment:
        return Shipment(result, self.client)

    def get_resource_path(self) -> str:
        return f"orders/{self._order.id}/shipments"

    def create(self, data: Optional[Dict[str, Any]] = None, **params: Any) -> Shipment:
        """Create a shipment for an order.

        If the data parameter is omitted, a shipment for all order lines is assumed.
        """
        if data is None:
            data = {"lines": []}
        return super().create(data, **params)

    def get(self, resource_id: str, **params: Any) -> Shipment:
        self.validate_resource_id(resource_id, "shipment ID")
        return super().get(resource_id, **params)

    def update(self, resource_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> Shipment:
        self.validate_resource_id(resource_id, "shipment ID")
        return super().update(resource_id, data, **params)



from .captures import *  # noqa: F401, F403
from .chargebacks import *  # noqa: F401, F403
from .clients import *  # noqa: F401, F403
from .customers import *  # noqa: F401, F403
from .invoices import *  # noqa: F401, F403
from .mandates import *  # noqa: F401, F403
from .methods import *  # noqa: F401, F403
from .onboarding import *  # noqa: F401, F403
from .order_lines import *  # noqa: F401, F403
from .orders import *  # noqa: F401, F403
from .organizations import *  # noqa: F401, F403
from .payment_links import *  # noqa: F401, F403
from .payments import *  # noqa: F401, F403
from .permissions import *  # noqa: F401, F403
from .profiles import *  # noqa: F401, F403
from .refunds import *  # noqa: F401, F403
from .settlements import *  # noqa: F401, F403
from .shipments import *  # noqa: F401, F403
from .subscriptions import *  # noqa: F401, F403



import re

from .base import ObjectBase
from .customer import Customer


class Subscription(ObjectBase):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources import CustomerSubscriptions

        return CustomerSubscriptions(client)

    STATUS_ACTIVE = "active"
    STATUS_PENDING = "pending"  # Waiting for a valid mandate.
    STATUS_CANCELED = "canceled"
    STATUS_SUSPENDED = "suspended"  # Active, but mandate became invalid.
    STATUS_COMPLETED = "completed"

    @property
    def status(self):
        return self._get_property("status")

    def is_active(self):
        return self.status == self.STATUS_ACTIVE

    def is_pending(self):
        return self.status == self.STATUS_PENDING

    def is_canceled(self):
        return self.status == self.STATUS_CANCELED

    def is_suspended(self):
        return self.status == self.STATUS_SUSPENDED

    def is_completed(self):
        return self.status == self.STATUS_COMPLETED

    @property
    def resource(self):
        return self._get_property("resource")

    @property
    def id(self):
        return self._get_property("id")

    @property
    def mode(self):
        return self._get_property("mode")

    @property
    def created_at(self):
        return self._get_property("createdAt")

    @property
    def amount(self):
        return self._get_property("amount")

    @property
    def times(self):
        return int(self._get_property("times"))

    @property
    def times_remaining(self):
        return int(self._get_property("timesRemaining"))

    @property
    def interval(self):
        return self._get_property("interval")

    @property
    def start_date(self):
        return self._get_property("startDate")

    @property
    def next_payment_date(self):
        return self._get_property("nextPaymentDate")

    @property
    def description(self):
        return self._get_property("description")

    @property
    def method(self):
        return self._get_property("method")

    @property
    def mandate_id(self):
        return self._get_property("mandateId")

    @property
    def canceled_at(self):
        return self._get_property("canceledAt")

    @property
    def webhook_url(self):
        return self._get_property("webhookUrl")

    @property
    def metadata(self):
        return self._get_property("metadata")

    @property
    def application_fee(self):
        return self._get_property("applicationFee")

    def get_customer(self):
        """Return the customer for this subscription."""
        url = self._get_link("customer")
        return self.client.customers.from_url(url)

    @property
    def customer_id(self):
        """
        Retrieve the customer id from the customer link.

        The customer_id is not available as a direct subscription property,
        but we need it to implement various features that need it. The only
        option is to extract it from the link.
        """
        url = self._get_link("customer")
        matches = re.findall(r"/customers/(cst_\w+)", url)
        if matches:
            return matches[0]

    def get_profile(self):
        """Return the profile related to this subscription."""
        url = self._get_link("profile")
        if not url:
            return None
        return self.client.profiles.from_url(url)

    def get_mandate(self):
        if self.mandate_id and self.customer_id:
            from ..resources import CustomerMandates

            customer = Customer({"id": self.customer_id}, self.client)
            return CustomerMandates(self.client, customer).get(self.mandate_id)

    @property
    def payments(self):
        # We could also have implemented this using the "payments" entry from the _links, but then we would not have
        # the explicit interface using .payments.list()
        from ..resources import SubscriptionPayments

        customer = Customer({"id": self.customer_id}, self.client)
        return SubscriptionPayments(self.client, customer=customer, subscription=self)



from .base import ObjectBase


class Capture(ObjectBase):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources import PaymentCaptures

        return PaymentCaptures(client)

    @property
    def id(self):
        return self._get_property("id")

    @property
    def mode(self):
        return self._get_property("mode")

    @property
    def amount(self):
        return self._get_property("amount")

    @property
    def settlement_amount(self):
        return self._get_property("settlementAmount")

    @property
    def payment_id(self):
        return self._get_property("paymentId")

    @property
    def shipment_id(self):
        return self._get_property("shipmentId")

    @property
    def settlement_id(self):
        return self._get_property("settlementId")

    @property
    def created_at(self):
        return self._get_property("createdAt")

    def get_payment(self):
        """Return the payment for this capture."""
        # TODO Use the embedded payment data, if available.
        return self.client.payments.get(self.payment_id)

    def get_shipment(self):
        """Return the shipment for this capture."""
        url = self._get_link("shipment")
        if url:
            from ..resources import OrderShipments

            # We fake the order object here, since it is not used by from_url()
            return OrderShipments(self.client, order=None).from_url(url)

    def get_settlement(self):
        """Return the settlement for this capture."""
        return self.client.settlements.get(self.settlement_id)



from typing import Any, Dict, Optional

from ..objects.customer import Customer
from .base import ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin


class Customers(ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin):
    """Resource handler for the `/customers` endpoint."""

    RESOURCE_ID_PREFIX: str = "cst_"

    def get_resource_object(self, result: dict) -> Customer:
        return Customer(result, self.client)

    def get(self, resource_id: str, **params: Any) -> Customer:
        self.validate_resource_id(resource_id, "customer ID")
        return super().get(resource_id, **params)

    def update(self, resource_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> Customer:
        self.validate_resource_id(resource_id, "customer ID")
        return super().update(resource_id, data, **params)

    def delete(self, resource_id: str, **params: Any) -> dict:
        self.validate_resource_id(resource_id, "customer ID")
        return super().delete(resource_id, **params)



from typing import Any, Dict, Optional

from ..objects.order import Order
from .base import ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin

__all__ = [
    "Orders",
]


class Orders(ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin):
    """Resource handler for the `/orders` endpoint."""

    RESOURCE_ID_PREFIX: str = "ord_"

    def get_resource_object(self, result: dict) -> Order:
        return Order(result, self.client)

    def get(self, resource_id: str, **params: Any) -> Order:
        self.validate_resource_id(resource_id, "order ID")
        return super().get(resource_id, **params)

    def delete(self, resource_id: str, **params: Any) -> dict:
        """Cancel order and return the order object.

        Deleting an order causes the order status to change to canceled.
        The updated order object is returned.
        """
        self.validate_resource_id(resource_id, "order ID")
        result = super().delete(resource_id, **params)
        return self.get_resource_object(result)

    def update(self, resource_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> Order:
        """Update an order, and return the updated order."""
        self.validate_resource_id(resource_id, "order ID")
        return super().update(resource_id, data, **params)



from typing import TYPE_CHECKING, Any, Dict, Optional

from ..objects.customer import Customer
from ..objects.subscription import Subscription
from .base import (
    ResourceBase,
    ResourceCreateMixin,
    ResourceDeleteMixin,
    ResourceGetMixin,
    ResourceListMixin,
    ResourceUpdateMixin,
)

if TYPE_CHECKING:
    from ..client import Client

__all__ = [
    "CustomerSubscriptions",
    "Subscriptions",
]


class SubscriptionsBase(ResourceBase):
    RESOURCE_ID_PREFIX: str = "sub_"

    def get_resource_object(self, result: dict) -> Subscription:
        return Subscription(result, self.client)


class Subscriptions(SubscriptionsBase, ResourceListMixin):
    """Resource handler for the `/subscriptions` endpoint."""

    pass


class CustomerSubscriptions(
    SubscriptionsBase,
    ResourceCreateMixin,
    ResourceDeleteMixin,
    ResourceGetMixin,
    ResourceListMixin,
    ResourceUpdateMixin,
):
    """Resource handler for the `/customers/:customer_id:/subscriptions` endpoint."""

    _customer: Customer

    def __init__(self, client: "Client", customer: Customer) -> None:
        self._customer = customer
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"customers/{self._customer.id}/subscriptions"

    def get(self, resource_id: str, **params: Any) -> Subscription:
        self.validate_resource_id(resource_id, "subscription ID")
        return super().get(resource_id, **params)

    def update(self, resource_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> Subscription:
        self.validate_resource_id(resource_id, "subscription ID")
        return super().update(resource_id, data, **params)

    def delete(self, resource_id: str, **params: Any) -> dict:
        self.validate_resource_id(resource_id, "subscription ID")
        resp = super().delete(resource_id, **params)
        return self.get_resource_object(resp)



from .base import ObjectBase


class Issuer(ObjectBase):
    @property
    def resource(self):
        return self._get_property("resource")

    @property
    def id(self):
        return self._get_property("id")

    @property
    def name(self):
        return self._get_property("name")

    @property
    def image_svg(self):
        images = self._get_property("image")
        return images["svg"]

    @property
    def image_size1x(self):
        images = self._get_property("image")
        return images["size1x"]

    @property
    def image_size2x(self):
        images = self._get_property("image")
        return images["size2x"]



from ..resources import OrderLines
from .base import ObjectBase


class Order(ObjectBase):
    def __init__(self, data, client):
        super().__init__(data, client)

    @classmethod
    def get_resource_class(cls, client):
        from ..resources import Orders

        return Orders(client)

    STATUS_CREATED = "created"
    STATUS_PAID = "paid"
    STATUS_AUTHORIZED = "authorized"
    STATUS_CANCELED = "canceled"
    STATUS_SHIPPING = "shipping"
    STATUS_COMPLETED = "completed"
    STATUS_EXPIRED = "expired"

    @property
    def id(self):
        return self._get_property("id")

    @property
    def resource(self):
        return self._get_property("resource")

    @property
    def profile_id(self):
        return self._get_property("profileId")

    @property
    def method(self):
        return self._get_property("method")

    @property
    def mode(self):
        return self._get_property("mode")

    @property
    def amount(self):
        return self._get_property("amount")

    @property
    def amount_captured(self):
        return self._get_property("amountCaptured")

    @property
    def amount_refunded(self):
        return self._get_property("amountRefunded")

    @property
    def status(self):
        return self._get_property("status")

    @property
    def is_cancelable(self):
        return self._get_property("isCancelable")

    @property
    def billing_address(self):
        return self._get_property("billingAddress")

    @property
    def consumer_date_of_birth(self):
        return self._get_property("consumerDateOfBirth")

    @property
    def order_number(self):
        return self._get_property("orderNumber")

    @property
    def shipping_address(self):
        return self._get_property("shippingAddress")

    @property
    def locale(self):
        return self._get_property("locale")

    @property
    def metadata(self):
        return self._get_property("metadata")

    @property
    def redirect_url(self):
        return self._get_property("redirectUrl")

    @property
    def webhook_url(self):
        return self._get_property("webhookUrl")

    @property
    def created_at(self):
        return self._get_property("createdAt")

    @property
    def expires_at(self):
        return self._get_property("expiresAt")

    @property
    def expired_at(self):
        return self._get_property("expiredAt")

    @property
    def paid_at(self):
        return self._get_property("paidAt")

    @property
    def authorized_at(self):
        return self._get_property("authorizedAt")

    @property
    def canceled_at(self):
        return self._get_property("canceledAt")

    @property
    def completed_at(self):
        return self._get_property("completedAt")

    # documented _links

    @property
    def checkout_url(self):
        return self._get_link("checkout")

    # additional methods

    def is_created(self):
        return self._get_property("status") == self.STATUS_CREATED

    def is_paid(self):
        return self._get_property("status") == self.STATUS_PAID

    def is_authorized(self):
        return self._get_property("status") == self.STATUS_AUTHORIZED

    def is_canceled(self):
        return self._get_property("status") == self.STATUS_CANCELED

    def is_shipping(self):
        return self._get_property("status") == self.STATUS_SHIPPING

    def is_completed(self):
        return self._get_property("status") == self.STATUS_COMPLETED

    def is_expired(self):
        return self._get_property("status") == self.STATUS_EXPIRED

    def has_refunds(self):
        return self.amount_refunded is not None

    @property
    def refunds(self):
        from ..resources import OrderRefunds

        return OrderRefunds(self.client, self)

    @property
    def lines(self):
        return OrderLines(self.client, self)

    @property
    def shipments(self):
        from ..resources import OrderShipments

        return OrderShipments(self.client, self)

    @property
    def payments(self):
        from ..resources import OrderPayments

        return OrderPayments(self.client, self)



from typing import Any, Dict, Optional

from ..objects.profile import Profile
from .base import ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin

__all__ = [
    "Profiles",
]


class Profiles(ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin):
    """Resource handler for the `/profiles` endpoint."""

    RESOURCE_ID_PREFIX: str = "pfl_"

    def get_resource_object(self, result: dict) -> Profile:
        return Profile(result, self.client)

    def get(self, resource_id: str, **params: Any) -> Profile:
        if resource_id != "me":
            self.validate_resource_id(resource_id, "profile ID")
        return super().get(resource_id, **params)

    def delete(self, resource_id: str, **params: Any) -> dict:
        self.validate_resource_id(resource_id, "profile ID")
        return super().delete(resource_id, **params)

    def update(self, resource_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> Profile:
        self.validate_resource_id(resource_id, "profile ID")
        return super().update(resource_id, **params)



from typing import TYPE_CHECKING, Any, Dict, List, Optional

from ..error import IdentifierError
from ..objects.issuer import Issuer
from ..objects.list import ObjectList
from ..objects.method import Method
from .base import ResourceBase, ResourceGetMixin, ResourceListMixin

if TYPE_CHECKING:
    from ..client import Client
    from ..objects.profile import Profile


__all__ = [
    "Methods",
    "ProfileMethods",
]


class MethodsBase(ResourceBase):
    def get_resource_object(self, result: dict) -> Method:
        return Method(result, self.client)


class Methods(MethodsBase, ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/methods` endpoint."""

    def all(self, **params: Any) -> ObjectList:
        """List all mollie payment methods, including methods that aren't activated in your profile."""
        resource_path = self.get_resource_path()
        path = f"{resource_path}/all"
        result = self.perform_api_call(self.REST_LIST, path, params=params)
        return ObjectList(result, Method, self.client)


class ProfileMethods(MethodsBase):
    """Resource handler for the `/profiles/:profile_id:/methods` endpoint."""

    _profile: "Profile"

    # A list of payment methods that requires management of specific issuers
    PAYMENT_METHODS_REQUIRING_ISSUER: List[str] = [
        Method.GIFTCARD,
        Method.VOUCHER,
    ]

    def __init__(self, client: "Client", profile: "Profile") -> None:
        self._profile = profile
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"profiles/{self._profile.id}/methods"

    def enable(self, method_id: str, **params: Any) -> Method:
        """
        Enable payment method for profile.

        For vouchers and giftcards, you need to enable specific issuers using enable_issuer().

        :param method_id: The payment method to enable.
        :type method_id: str
        """
        if method_id in self.PAYMENT_METHODS_REQUIRING_ISSUER:
            raise IdentifierError(f"Cannot enable '{method_id}' method, it requires enabling specific issuers.")

        resource_path = self.get_resource_path()
        path = f"{resource_path}/{method_id}"
        result = self.perform_api_call(self.REST_CREATE, path, params=params)
        return self.get_resource_object(result)

    def disable(self, method_id: str, **params: Any) -> Method:
        """
        Disable payment method for the profile.

        For vouchers and giftcards, you need to enable specific issuers using disable_issuer().

        :param method_id: The payment method to disable.
        :type method_id: str
        """
        if method_id in self.PAYMENT_METHODS_REQUIRING_ISSUER:
            raise IdentifierError(f"Cannot disable '{method_id}' method, it requires disabling specific issuers.")

        resource_path = self.get_resource_path()
        path = f"{resource_path}/{method_id}"
        result = self.perform_api_call(self.REST_DELETE, path, params=params)
        return self.get_resource_object(result)

    def list(self, **params: Any) -> ObjectList:
        """List the payment methods for the profile."""
        params.update({"profileId": self._profile.id})
        # Divert the API call to the general Methods resource
        return Methods(self.client).list(**params)

    def enable_issuer(
        self, method_id: str, issuer_id: str, data: Optional[Dict[str, Any]] = None, **params: Any
    ) -> Issuer:
        """
        Enable an issuer for a payment method.

        Vouchers and giftcards need to enable specific issuers, in stead of enabling the payment method in general.

        :param method_id: The payment method, one of: voucher, giftcard.
        :type method_id: str
        :param issuer_id: The issuer to enable.
        :type issuer_id: str
        :param data: Optional payload
        """
        if method_id not in self.PAYMENT_METHODS_REQUIRING_ISSUER:
            raise IdentifierError(f"Payment method '{method_id}' does not support issuers.")

        resource_path = self.get_resource_path()
        path = f"{resource_path}/{method_id}/issuers/{issuer_id}"
        result = self.perform_api_call(self.REST_CREATE, path, data, params)
        return Issuer(result, self.client)

    def disable_issuer(self, method_id: str, issuer_id: str, **params: Any) -> Issuer:
        """
        Disable an issuer for a payment method.

        Vouchers and giftcards need to disable specific issuers, in stead of disabling the payment method in general.

        :param method_id: The payment method, one of: voucher, giftcard.
        :type method_id: str
        :param issuer_id: The issuer to enable.
        :type issuer_id: str
        """
        if method_id not in self.PAYMENT_METHODS_REQUIRING_ISSUER:
            raise IdentifierError(f"Payment method '{method_id}' does not support issuers.")

        resource_path = self.get_resource_path()
        path = f"{resource_path}/{method_id}/issuers/{issuer_id}"
        result = self.perform_api_call(self.REST_DELETE, path, params=params)
        return Issuer(result, self.client)



from typing import TYPE_CHECKING, Any, Dict, Optional

from ..objects.customer import Customer
from ..objects.list import ObjectList
from ..objects.order import Order
from ..objects.payment import Payment
from ..objects.profile import Profile
from ..objects.subscription import Subscription
from .base import (
    ResourceBase,
    ResourceCreateMixin,
    ResourceDeleteMixin,
    ResourceGetMixin,
    ResourceListMixin,
    ResourceUpdateMixin,
)

if TYPE_CHECKING:
    from ..client import Client
    from ..objects.settlement import Settlement

__all__ = [
    "CustomerPayments",
    "OrderPayments",
    "Payments",
    "ProfilePayments",
    "SettlementPayments",
    "SubscriptionPayments",
]


class PaymentsBase(ResourceBase):
    RESOURCE_ID_PREFIX: str = "tr_"

    def get_resource_object(self, result: dict) -> Payment:
        from ..objects.payment import Payment

        return Payment(result, self.client)


class Payments(
    PaymentsBase, ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin, ResourceUpdateMixin
):
    """Resource handler for the `/payments` endpoint."""

    def get(self, resource_id: str, **params: Any) -> Payment:
        self.validate_resource_id(resource_id, "payment ID")
        return super().get(resource_id, **params)

    def delete(self, resource_id: str, **params: Any) -> dict:
        """Cancel payment and return the payment object.

        Deleting a payment causes the payment status to change to canceled.
        The updated payment object is returned.
        """
        self.validate_resource_id(resource_id, "payment ID")
        result = super().delete(resource_id, **params)
        return self.get_resource_object(result)

    def update(self, resource_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> Payment:
        self.validate_resource_id(resource_id, "payment ID")
        return super().update(resource_id, data, **params)


class OrderPayments(PaymentsBase, ResourceCreateMixin):
    """Resource handler for the `/orders/:order_id:/payments` endpoint."""

    _order: Order

    def __init__(self, client: "Client", order: Order) -> None:
        self._order = order
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"orders/{self._order.id}/payments"

    def list(self) -> ObjectList:
        """
        List the payments that might have been embedded in the related order.

        When you receive an EmbedError, you need to embed the payments in the parent order.
        """
        payments = self._order.get_embedded("payments")

        data = {
            "_embedded": {
                "payments": payments,
            },
            "count": len(payments),
        }
        return ObjectList(data, Payment, self.client)


class CustomerPayments(PaymentsBase, ResourceCreateMixin, ResourceListMixin):
    """Resource handler for the `/customers/:customer_id:/payments` endpoint."""

    _customer: Customer

    def __init__(self, client: "Client", customer: Customer) -> None:
        self._customer = customer
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"customers/{self._customer.id}/payments"


class SubscriptionPayments(PaymentsBase, ResourceListMixin):
    """Resource handler for the `/customers/:customer_id:/subscriptions/:subscription_id:/payments` endpoint."""

    _customer: Customer
    _subscription: Subscription

    def __init__(self, client: "Client", customer: Customer, subscription: Subscription) -> None:
        self._customer = customer
        self._subscription = subscription
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"customers/{self._customer.id}/subscriptions/{self._subscription.id}/payments"


class SettlementPayments(PaymentsBase, ResourceListMixin):
    """Resource handler for the `/settlements/:settlement_id:/payments` endpoint."""

    _settlement: "Settlement"

    def __init__(self, client: "Client", settlement: "Settlement") -> None:
        self._settlement = settlement
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"settlements/{self._settlement.id}/payments"


class ProfilePayments(PaymentsBase):
    """
    Resource handler for the `/payments?profileId=:profile_id:` endpoint.

    This is separate from the `Payments` resource handler to make it easier to inject the profileId.
    """

    _profile: Profile

    def __init__(self, client: "Client", profile: Profile) -> None:
        self._profile = profile
        super().__init__(client)

    def list(self, **params: Any) -> ObjectList:
        # Set the profileId in the query params
        params.update({"profileId": self._profile.id})
        return Payments(self.client).list(**params)



from typing import Any

from ..objects.organization import Organization
from .base import ResourceGetMixin

__all__ = [
    "Organizations",
]


class Organizations(ResourceGetMixin):
    """Resource handler for the `/organizations` endpoint."""

    RESOURCE_ID_PREFIX: str = "org_"

    def get_resource_object(self, result: dict) -> Organization:
        return Organization(result, self.client)

    def get(self, resource_id: str, **params: Any) -> Organization:
        if resource_id != "me":
            self.validate_resource_id(resource_id, "organization ID")
        return super().get(resource_id, **params)
